@@ -1,43 +1,8 @@
-/*	$Id: gmt_mgg_header2.c,v 1.22 2007/03/29 02:51:54 pwessel Exp $
+/*	$Id: gmt_mgg_header2.c,v 1.28 2008/04/03 20:01:18 guru Exp $
  *
  *	Code donated by David Divens, NOAA/NGDC
  *	Distributed under the GNU Public License (see COPYING for details)
- *	This is the README file:
- ------------------------------------------------------------------------
- Adding MGG file format to GMT
------------------------------
-1) Modify gmt_grdio.h
-   - Increment GMT_N_GRD_FORMATS.
-     This number is one based, the formats are zero based so this
-     value will be one greater than the grid number you are creating.
-     
-2) Modify gmt_customio.c
-   - Make sure both gmt_mgg_header2.h and gmt_mgg_header2.c are in the same
-     directory as gmt_customio.c
-
-   - Scroll down to the comment
-     FORMAT # N:
-
-   - Add the following lines:
-    id = N;
-    GMT_io_readinfo[id]  = (PFI) mgg2_read_grd_info;
-    GMT_io_writeinfo[id] = (PFI) mgg2_write_grd_info;
-    GMT_io_readgrd[id]   = (PFI) mgg2_read_grd;
-    GMT_io_writegrd[id]  = (PFI) mgg2_write_grd;
-
-   - Jump down to the end of the file and add this line:
-    #include "gmt_mgg_header2.c"
-
-2) Modify gmt_customio.h
-   - At the end of this file add:
-    #include "gmt_mgg_header2."
-
-Build GMT with new format by typing:
-> make all
-
-Test format by checking a grid:
-> grdinfo gnaXXXXX.g03=N
---------------------------------------------------------------------*/
+ *--------------------------------------------------------------------*/
 
 /* Comments added by P. Wessel:
  *
@@ -83,7 +48,7 @@ int GMT2MGG2(struct GRD_HEADER *gmt, MGG_GRID_HEADER_2 *mgg)
 	double f;
 	memset(mgg, 0, sizeof(MGG_GRID_HEADER_2));
 	
-	mgg->version     = MGG_MAGIC_NUM + VERSION;
+	mgg->version     = MGG_MAGIC_NUM + MGG_VERSION;
 	mgg->length      = sizeof(MGG_GRID_HEADER_2);
 	mgg->dataType    = 1;
 	
@@ -203,23 +168,24 @@ static void swap_header(MGG_GRID_HEADER_2 *header)
    for (i = 0; i < GRD98_N_UNUSED; i++) swap_long(&header->unused[i]);
 }
 
-int GMT_is_mgg2_grid (char *file)
+int GMT_is_mgg2_grid (struct GRD_HEADER *header)
 {	/* Determine if file is a GRD98 file */
 	FILE *fp = NULL;
 	MGG_GRID_HEADER_2 mggHeader;
 
-	if (!strcmp(file, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
-	if ((fp = GMT_fopen(file, GMT_io.r_mode)) == NULL) return (GMT_GRDIO_OPEN_FAILED);
+	if (!strcmp(header->name, "=")) return (GMT_GRDIO_PIPE_CODECHECK);	/* Cannot check on pipes */
+	if ((fp = GMT_fopen(header->name, GMT_io.r_mode)) == NULL) return (GMT_GRDIO_OPEN_FAILED);
 
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
-	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
+	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
 	/* Swap header bytes if necessary */
 	swap_header(&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + VERSION) return (-1);	/* Not this kind of file */
-	return (GMT_grd_format_decoder ("rf"));
+	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) return (-1);	/* Not this kind of file */
+	header->type = GMT_grd_format_decoder ("rf");
+	return (header->type);
 }
 
 int mgg2_read_grd_info (struct GRD_HEADER *header)
@@ -233,14 +199,14 @@ int mgg2_read_grd_info (struct GRD_HEADER *header)
 		return (GMT_GRDIO_OPEN_FAILED);
 
 	memset(&mggHeader, '\0', sizeof(MGG_GRID_HEADER_2));
-	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
+	if (GMT_fread(&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_READ_FAILED);
 
 	/* Swap header bytes if necessary */
 	swap_header(&mggHeader);
 
 	/* Check the magic number and size of header */
-	if (mggHeader.version < MGG_MAGIC_NUM + VERSION) {
-		fprintf(stderr, "GMT Fatal Error: Unrecognized header, expected 0x%04X saw 0x%04X\n", MGG_MAGIC_NUM + VERSION, mggHeader.version);
+	if (mggHeader.version < MGG_MAGIC_NUM + MGG_VERSION) {
+		fprintf(stderr, "GMT Fatal Error: Unrecognized header, expected 0x%04X saw 0x%04X\n", MGG_MAGIC_NUM + MGG_VERSION, mggHeader.version);
 		return (GMT_GRDIO_GRD98_BADMAGIC);
 	}
 
@@ -272,7 +238,7 @@ int mgg2_write_grd_info (struct GRD_HEADER *header)
 	/* Swap header bytes if necessary */
 	swap_header(&mggHeader);
 
-	if (GMT_fwrite (&mggHeader, sizeof(MGG_GRID_HEADER_2), 1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
+	if (GMT_fwrite (&mggHeader, sizeof(MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 
 	if (fp != GMT_stdout) GMT_fclose (fp);
 	
@@ -286,8 +252,9 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	int  *tLong  = NULL;
 	short *tShort = NULL;
 	char  *tChar  = NULL;
-	int first_col, last_col, first_row, last_row, nm, kk, one_or_zero;
-	int i, j, j2, ij, width_in, width_out, height_in, i_0_out, inc = 1;
+	GMT_LONG first_col, last_col, first_row, last_row, one_or_zero;
+	GMT_LONG j, j2, width_in, height_in, i_0_out, inc = 1;
+	GMT_LONG i, kk, ij, nm, width_out;
 	BOOLEAN piping = FALSE, geo = FALSE;
 	double half_or_zero, x, small;
 	long long_offset;	/* For fseek only */
@@ -299,7 +266,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		piping = TRUE;
 	}
 	else if ((fp = GMT_fopen (header->name, GMT_io.r_mode)) != NULL) {
-		if (GMT_fread(&mggHeader, 1, sizeof(MGG_GRID_HEADER_2), fp) != 1) return (GMT_GRDIO_READ_FAILED);
+		if (GMT_fread(&mggHeader, (size_t)1, sizeof(MGG_GRID_HEADER_2), fp) != 1) return (GMT_GRDIO_READ_FAILED);
 		swap_header(&mggHeader);
 		if (mggHeader.numType == 0) mggHeader.numType = sizeof(int);
 	}
@@ -309,12 +276,12 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	}
 	
 	if (w == 0.0 && e == 0.0) {	/* Get entire file and return */
-		nm = header->nx * header->ny;
-		tLong  = (int *) GMT_memory (CNULL, nm, sizeof (int), "mgg_read_grd");
+		nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
+		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
 
-		if (GMT_fread ((void *)tLong, mggHeader.numType, nm, fp) != (size_t)nm) return (GMT_GRDIO_READ_FAILED);
+		if (GMT_fread ((void *)tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_READ_FAILED);
 		for (i = 0; i < nm; i++) {
 			/* 4-byte values */
 			if (mggHeader.numType == sizeof(int)) {
@@ -360,18 +327,18 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	/* Get first and last row and column numbers */
 	
 	small = 0.1 * header->x_inc;
-	first_col = (int)floor ((w - header->x_min + small) / header->x_inc);
-	last_col  = (int)ceil ((e - header->x_min - small) / header->x_inc) - 1 + one_or_zero;
+	first_col = (GMT_LONG)floor ((w - header->x_min + small) / header->x_inc);
+	last_col  = (GMT_LONG)ceil ((e - header->x_min - small) / header->x_inc) - 1 + one_or_zero;
 	small = 0.1 * header->y_inc;
-	first_row = (int)floor ((header->y_max - n + small) / header->y_inc);
-	last_row  = (int)ceil ((header->y_max - s - small) / header->y_inc) - 1 + one_or_zero;
+	first_row = (GMT_LONG)floor ((header->y_max - n + small) / header->y_inc);
+	last_row  = (GMT_LONG)ceil ((header->y_max - s - small) / header->y_inc) - 1 + one_or_zero;
 
 	if ((last_col - first_col + 1) > width_in) last_col--;
 	if ((last_row - first_row + 1) > height_in) last_row--;
 	if ((last_col - first_col + 1) > width_in) first_col++;
 	if ((last_row - first_row + 1) > height_in) first_row++;
 	
-	width_out = width_in;		/* Width of output array */
+	width_out = (GMT_LONG)width_in;		/* Width of output array */
 	if (pad[0] > 0) width_out += pad[0];
 	if (pad[1] > 0) width_out += pad[1];
 	
@@ -379,11 +346,11 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	
 	if (geo) {	/* Must rollover in longitudes */
 		int *k;
-		tLong  = (int *) GMT_memory (CNULL, header->nx, sizeof (int), "mgg_read_grd");
+		tLong  = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *)tLong;
 		tChar  = (char *)tLong;
 
-		k = (int *) GMT_memory (CNULL, width_in, sizeof (int), "mgg_read_grd");
+		k = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_read_grd");
 		
 		half_or_zero = (header->node_offset) ? 0.5 : 0.0;
 		small = 0.1 * header->x_inc;	/* Anything smaller than 0.5 dx will do */
@@ -397,7 +364,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else { /* Simply seek by it */
 			long_offset = (long)(first_row * header->nx * mggHeader.numType);
@@ -405,7 +372,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
-			if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			ij = (j2 + pad[3]) * width_out + i_0_out;
 			for (i = 0; i < width_in; i++) {
@@ -434,20 +401,20 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		}
 		if (piping)	{ /* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 			
 		GMT_free ((void *)k);
 	}
 	else {	/* A bit easier here */
-		tLong = (int *) GMT_memory (CNULL, header->nx, sizeof (int), "mgg_read_grd");
+		tLong = (int *) GMT_memory (CNULL, (size_t)header->nx, sizeof (int), "mgg_read_grd");
 		tShort = (short *) tLong;
 		tChar  = (char *) tLong;
 
 		if (piping)	{ /* Skip data by reading it */
 			for (j = 0; j < first_row; j++) {
-				if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		} else {/* Simply seek by it */
 			long_offset = (long) (first_row * header->nx * mggHeader.numType);
@@ -456,7 +423,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 		
 		for (j = first_row, j2 = 0; j <= last_row; j++, j2++) {
 			ij = (j2 + pad[3]) * width_out + i_0_out;
-			if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+			if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) != (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			
 			for (i = 0; i < width_in; i++) {
 				kk = ij+i;
@@ -488,7 +455,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 
 		if (piping)	{/* Skip data by reading it */
 			for (j = last_row + 1; j < header->ny; j++) {
-				if (GMT_fread ((void *) tLong, mggHeader.numType, header->nx, fp) < (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
+				if (GMT_fread ((void *) tLong, (size_t)mggHeader.numType, (size_t)header->nx, fp) < (size_t)header->nx) return (GMT_GRDIO_READ_FAILED);
 			}
 		}
 	}
@@ -504,7 +471,7 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 	for (j = 0; j < header->ny; j++) {
 		for (i = 0; i < header->nx; i++) {
 			ij = (j + pad[3]) * width_out + i + pad[0];
-			if (GMT_is_fnan ((double) grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
@@ -519,9 +486,11 @@ int mgg2_read_grd (struct GRD_HEADER *header, float *grid, double w, double e, d
 int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
 {
 	MGG_GRID_HEADER_2	mggHeader;
-	int i, i2, nm, kk, err, *k;
-	int j, ij, j2, width_in, width_out, height_out, one_or_zero;
-	int first_col, last_col, first_row, last_row;
+	int *k;
+	GMT_LONG i2, kk, err;
+	GMT_LONG j, j2, width_out, height_out, one_or_zero;
+	GMT_LONG first_col, last_col, first_row, last_row;
+	GMT_LONG i, nm, ij, width_in;
 	
 	BOOLEAN geo = FALSE;
 	double half_or_zero, small, x;
@@ -541,25 +510,25 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	if (w == 0.0 && e == 0.0) {	/* Write entire file and return */
 		/* Find min/max of data */
 		
-		nm = header->nx * header->ny;
+		nm = ((GMT_LONG)header->nx) * ((GMT_LONG)header->ny);
 		header->z_min = DBL_MAX;	header->z_max = -DBL_MAX;
 		for (i = 0; i < nm; i++) {
 			ij = (complex) ? 2 * i : i;
-			if (GMT_is_fnan ((double)grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
 		if ((err = GMT2MGG2(header, &mggHeader))) return (err);
 		swap_header(&mggHeader);
-		if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), 1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
+		if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 		swap_header(&mggHeader);
 
-		tLong  = (int *) GMT_memory (CNULL, nm, sizeof (int), "mgg_write_grd");
+		tLong  = (int *) GMT_memory (CNULL, (size_t)nm, sizeof (int), "mgg_write_grd");
 		tShort = (short*)tLong;
 		tChar  = (char*)tLong;
 		
 		for (i = 0; i < nm; i++) {
-			if (GMT_is_fnan ((double)grid[i])) {
+			if (GMT_is_fnan (grid[i])) {
 				if (mggHeader.numType == sizeof(int)) {
 			       tLong[i]  = mggHeader.nanValue;
 			       swap_long(&tLong[i]);
@@ -593,7 +562,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				}
 			}
 		}
-		if (GMT_fwrite (tLong, mggHeader.numType, nm, fp) != (size_t)nm) return (GMT_GRDIO_WRITE_FAILED);
+		if (GMT_fwrite (tLong, (size_t)mggHeader.numType, (size_t)nm, fp) != (size_t)nm) return (GMT_GRDIO_WRITE_FAILED);
 		GMT_free ((void *)tLong);
 		return (GMT_NOERROR);
 	}
@@ -610,18 +579,18 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	/* Get first and last row and column numbers */
 	
 	small     = 0.1 * header->x_inc;
-	first_col = (int)floor ((w - header->x_min + small) / header->x_inc);
-	last_col  = (int)ceil ((e - header->x_min - small) / header->x_inc) -1 + one_or_zero;
+	first_col = (GMT_LONG)floor ((w - header->x_min + small) / header->x_inc);
+	last_col  = (GMT_LONG)ceil ((e - header->x_min - small) / header->x_inc) -1 + one_or_zero;
 	small     = 0.1 * header->y_inc;
-	first_row = (int)floor ((header->y_max - n + small) / header->y_inc);
-	last_row  = (int)ceil ((header->y_max - s - small) / header->y_inc) -1 + one_or_zero;
+	first_row = (GMT_LONG)floor ((header->y_max - n + small) / header->y_inc);
+	last_row  = (GMT_LONG)ceil ((header->y_max - s - small) / header->y_inc) -1 + one_or_zero;
 	
 	if ((last_col - first_col + 1) > width_out) last_col--;
 	if ((last_row - first_row + 1) > height_out) last_row--;
 	if ((last_col - first_col + 1) > width_out) first_col++;
 	if ((last_row - first_row + 1) > height_out) first_row++;
 
-	width_in = width_out;		/* Physical width of input array */
+	width_in = (GMT_LONG)width_out;		/* Physical width of input array */
 	if (pad[0] > 0) width_in += pad[0];
 	if (pad[1] > 0) width_in += pad[1];
 	
@@ -636,7 +605,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	for (j = first_row, j2 = pad[3]; j <= last_row; j++, j2++) {
 		for (i = first_col, i2 = pad[0]; i <= last_col; i++, i2++) {
 			ij = j2 * width_in + i2;
-			if (GMT_is_fnan ((double)grid[ij])) continue;
+			if (GMT_is_fnan (grid[ij])) continue;
 			header->z_min = MIN (header->z_min, grid[ij]);
 			header->z_max = MAX (header->z_max, grid[ij]);
 		}
@@ -645,15 +614,15 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 	/* store header information and array */
 	if ((err = GMT2MGG2(header, &mggHeader))) return (err);;
 	swap_header(&mggHeader);
-	if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), 1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
+	if (GMT_fwrite (&mggHeader, sizeof (MGG_GRID_HEADER_2), (size_t)1, fp) != 1) return (GMT_GRDIO_WRITE_FAILED);
 	swap_header(&mggHeader);
 
-	tLong  = (int *) GMT_memory (CNULL, width_in, sizeof (int), "mgg_write_grd");
+	tLong  = (int *) GMT_memory (CNULL, (size_t)width_in, sizeof (int), "mgg_write_grd");
 	tShort = (short *) tLong;
 	tChar  = (char *)  tLong;
 	
 	if (geo) {
-		k = (int *) GMT_memory (CNULL, width_out, sizeof (int), "mgg_write_grd");
+		k = (int *) GMT_memory (CNULL, (size_t)width_out, sizeof (int), "mgg_write_grd");
 		
 		half_or_zero = (header->node_offset) ? 0.5 : 0.0;
 		small = 0.1 * header->x_inc;	/* Anything smaller than 0.5 dx will do */
@@ -670,7 +639,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 			ij = j2 * width_in + i2;
 			for (i = 0; i < width_out; i++) {
 				kk = ij+k[i];
-				if (GMT_is_fnan ((double)grid[kk])) {
+				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
@@ -699,7 +668,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
 				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			GMT_fwrite ((void *)tLong, mggHeader.numType, width_out, fp);
+			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
 		}
 		GMT_free ((void *)k);
 	}
@@ -709,7 +678,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 			ij = j2 * width_in + i2;
 			for (i = 0; i < width_out; i++) {
 				kk = ij+i;
-				if (GMT_is_fnan ((double)grid[kk])) {
+				if (GMT_is_fnan (grid[kk])) {
 					if (mggHeader.numType == sizeof(int))       tLong[i]  = mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(short)) tShort[i] = (short)mggHeader.nanValue;
 					else if (mggHeader.numType == sizeof(char))  tChar[i] = (char)mggHeader.nanValue;
@@ -737,7 +706,7 @@ int mgg2_write_grd (struct GRD_HEADER *header, float *grid, double w, double e,
 				if (mggHeader.numType == sizeof(int)) swap_long(&tLong[i]);
 				else if (mggHeader.numType == sizeof(short)) swap_word(&tShort[i]);
 			}
-			GMT_fwrite ((void *)tLong, mggHeader.numType, width_out, fp);
+			GMT_fwrite ((void *)tLong, (size_t)mggHeader.numType, (size_t)width_out, fp);
 		}
 	}
 	if (fp != GMT_stdout) GMT_fclose (fp);
