@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: gmtselect.c,v 1.84 2007/09/30 04:16:48 guru Exp $
+ *	$Id: gmtselect.c,v 1.91 2008/03/24 08:58:31 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -107,14 +107,15 @@ struct GMTSELECT_CTRL {	/* All control options for this program (except common a
 
 int main (int argc, char **argv)
 {
-	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, n_args, err;
+	int i, j, k, fno, n_files = 0, n_args, err;
 	int n_fields, n_expected_fields, ind, bin, last_bin = -1, n_output = -1;
 	int np[2], base = 3, wd[2], id, this_node, side, is_inside, row, col, pos;
+	
+	GMT_LONG n_read = 0, n_pass = 0;
 
-
-	BOOLEAN error = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, need_header;
+	BOOLEAN error = FALSE, dry_wet_only = FALSE, long_verbose = FALSE, need_header, shuffle;
 	BOOLEAN cartesian = FALSE, nofile = TRUE, done = FALSE, first = TRUE, inside, plain_xy;
-	BOOLEAN greenwich = FALSE, output_header = FALSE, do_project = FALSE, binary = FALSE;
+	BOOLEAN greenwich = FALSE, output_header = FALSE, do_project = FALSE, just_copy_record = FALSE;
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, xx, yy;
 	double *in, west_border = 0.0, east_border = 0.0;
@@ -344,8 +345,8 @@ int main (int argc, char **argv)
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t   Append l for long verbose, reporting every 1000 points.\n");
-		fprintf (stderr, "\t   -Z assumes the 3rd data column contains z-values and we want to keep records with\n");
-		fprintf (stderr, "\t      <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
+		fprintf (stderr, "\t-Z assumes the 3rd data column contains z-values and we want to keep records with\n");
+		fprintf (stderr, "\t   <min> <= z <= <max>.  Use - for <min> or <max> if there is no lower/upper limit.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
@@ -387,6 +388,8 @@ int main (int argc, char **argv)
 	GMT_setmode (GMT_OUT);
 #endif
 
+	shuffle = (gmtdefs.xy_toggle[GMT_IN] != gmtdefs.xy_toggle[GMT_OUT]);	/* Must rewrite output record */
+	
 	if (!project_info.region_supplied && Ctrl->N.active) {	/* If we use coastline data or used -fg but didnt give -R we imply -Rg */
 		project_info.region_supplied = TRUE;
 		west = 0.0;	east = 360.0;	south = -90.0;	north = +90.0;
@@ -394,7 +397,7 @@ int main (int argc, char **argv)
 		GMT_io.in_col_type[1] = GMT_IS_LAT;
 	}
 	if (project_info.region_supplied) {	/* -R was set directly or indirectly; hence must set -J if not supplied */
-		if (project_info.projection < 0) {	/* -J not specifed, set one implicitly */
+		if (project_info.projection == GMT_NO_PROJ) {	/* -J not specifed, set one implicitly */
 			if (GMT_io.in_col_type[0] == GMT_IS_LON) {	/* We know we have geographic data (either via -f or -N) */
 				project_info.degree[0] = project_info.degree[1] = TRUE;
 			}
@@ -433,7 +436,7 @@ int main (int argc, char **argv)
 		np[0] = np[1] = 0;
 	}
 
-	binary = (GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT]);
+	just_copy_record = !(GMT_io.binary[GMT_IN] || GMT_io.binary[GMT_OUT] || shuffle);
 
 	/* Initiate pointer to distance calculation function */
 	if (GMT_io.in_col_type[0] & GMT_IS_GEO && !do_project) {	/* Geographic data and no -R -J conversion */
@@ -579,12 +582,12 @@ int main (int argc, char **argv)
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 			if (GMT_io.status & GMT_IO_MISMATCH) {
-				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
 			n_read++;
-			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %d records, passed %d records\r", GMT_program, n_read, n_pass);
+			if (long_verbose && n_read%1000 == 0) fprintf (stderr, "%s: Read %ld records, passed %ld records\r", GMT_program, n_read, n_pass);
 
 			if (Ctrl->Z.active && n_expected_fields < 3) {
 				fprintf (stderr, "%s: -Z requires a data file with at least 3 columns; this file only has %d. Exiting.\n", GMT_program, n_fields);
@@ -659,6 +662,7 @@ int main (int argc, char **argv)
 					while (ind < c.nb && c.bins[ind] != bin) ind++;	/* Set ind to right bin */
 					if (ind == c.nb) continue;			/* Bin not among the chosen ones */
 					last_bin = bin;
+					GMT_free_shore (&c);	/* Free previously allocated arrays */
 					if ((err = GMT_get_shore_bin (ind, &c, Ctrl->A.area, Ctrl->A.low, Ctrl->A.high))) {
 						fprintf (stderr, "%s: %s [%s resolution shoreline]\n", GMT_program, GMT_strerror(err), shore_resolution[base]);
 						exit (EXIT_FAILURE);
@@ -721,22 +725,30 @@ int main (int argc, char **argv)
 				output_header = FALSE;
 			}
 
-			if (binary) {
+			if (just_copy_record)
+				fprintf (GMT_stdout, "%s", GMT_io.current_record);
+			else {
 				if (n_output < 0) n_output = n_expected_fields;
 				GMT_output (GMT_stdout, n_output, in);
 			}
-			else
-				fprintf (GMT_stdout, "%s", GMT_io.current_record);
 			n_pass++;
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
 
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %d records, passed %d records\n", GMT_program, n_read, n_pass);
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Read %ld records, passed %ld records\n", GMT_program, n_read, n_pass);
 
 	if (Ctrl->C.active) GMT_free_table (point);
 	if (Ctrl->L.active) GMT_free_table (line);
 	if (Ctrl->F.active) GMT_free_table (pol);
+	if (Ctrl->N.active) {
+		GMT_free_shore (&c);
+		GMT_shore_cleanup (&c);
+		for (id = 0; id < 2; id++) {
+			GMT_free_polygons (p[id], np[id]);
+			if (np[id]) GMT_free ((void *)p[id]);
+		}
+	}
 	
 	Free_Gmtselect_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -764,7 +776,7 @@ void *New_Gmtselect_Ctrl () {	/* Allocate and initialize a new control structure
 	int i;
 	struct GMTSELECT_CTRL *C;
 	
-	C = (struct GMTSELECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GMTSELECT_CTRL), "New_Gmtselect_Ctrl");
+	C = (struct GMTSELECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GMTSELECT_CTRL), "New_Gmtselect_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
