@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    $Id: grdblend.c,v 1.56 2007/09/30 04:16:48 guru Exp $
+ *    $Id: grdblend.c,v 1.63 2008/05/01 03:00:38 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -81,7 +81,7 @@ struct GRDBLEND_INFO {	/* Structure with info about each input grid file */
 
 int main (int argc, char **argv)
 {
-	int i, col, row, k, kk, m, n_blend, one_or_zero, err, error = 0;
+	int i, col, row, k, kk, m, n_blend, err, error = 0;
 	size_t n_fill, n_tot;
 	double wt_x, w, wt;
 	float *z, no_data_f;
@@ -239,13 +239,11 @@ int main (int argc, char **argv)
 
 	/* Initialize header structure for output blend grid */
 
-	one_or_zero = !S.header.node_offset;
-
 	S.header.z_min = S.header.z_max = 0.0;
 
 	GMT_err_fail (GMT_grd_RI_verify (&S.header, 1), Ctrl->G.file);
 
-	n_tot = (size_t)S.header.nx * (size_t)S.header.ny;
+	n_tot = ((size_t)S.header.nx) * ((size_t)S.header.ny);
 
 	z = (float *) GMT_memory (VNULL, (size_t)S.header.nx, sizeof (float), GMT_program);	/* Memory for one output row */
 
@@ -312,7 +310,10 @@ int main (int argc, char **argv)
 	if (!Ctrl->Q.active) GMT_err_fail (GMT_update_grd_info (Ctrl->G.file, &S.header), Ctrl->G.file);
 	GMT_free ((void *)z);
 
-	for (k = 0; k < n_blend; k++) if (blend[k].open) GMT_close_grd (&blend[k].G);	/* Close all input grd files still open */
+	for (k = 0; k < n_blend; k++) if (blend[k].open) {
+		GMT_free ((void *)blend[k].z);
+		GMT_close_grd (&blend[k].G);	/* Close all input grd files still open */
+	}
 
 	if (gmtdefs.verbose) {
 		char empty[GMT_TEXT_LEN];
@@ -324,7 +325,7 @@ int main (int argc, char **argv)
 				strcpy (empty, "NaN");
 			else
 				sprintf (empty, "%g", no_data_f);
-			fprintf (stderr, "%s: %d nodes assigned values, %d set to %s\n", GMT_program, (int)n_fill, (int)(n_tot - n_fill), empty);
+			fprintf (stderr, "%s: %ld nodes assigned values, %ld set to %s\n", GMT_program, (GMT_LONG)n_fill, (GMT_LONG)(n_tot - n_fill), empty);
 		}
 	}
 
@@ -368,17 +369,18 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		if (n == 0) {
 			h->node_offset = B[n].G.header.node_offset;
 			one_or_zero = !h->node_offset;
+			GMT_RI_prepare (h);	/* Ensure -R -I consistency and set nx, ny */
 		}
 		if (h->node_offset != B[n].G.header.node_offset){
-			fprintf (stderr, "%s: File %s has different registration that the first file given\n", GMT_program, B[n].file);
+			fprintf (stderr, "%s: File %s has different registration than the first file given\n", GMT_program, B[n].file);
 			exit (EXIT_FAILURE);
 		}
 		if (!GMT_IS_ZERO (B[n].G.header.x_inc - h->x_inc)){
-			fprintf (stderr, "%s: File %s has different x-increment (%g) that the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.x_inc, h->x_inc);
+			fprintf (stderr, "%s: File %s has different x-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.x_inc, h->x_inc);
 			exit (EXIT_FAILURE);
 		}
 		if (!GMT_IS_ZERO (B[n].G.header.y_inc - h->y_inc)){
-			fprintf (stderr, "%s: File %s has different y-increment (%g) that the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.y_inc, h->y_inc);
+			fprintf (stderr, "%s: File %s has different y-increment (%g) than the chosen output increment (%g)\n", GMT_program, B[n].file, B[n].G.header.y_inc, h->y_inc);
 			exit (EXIT_FAILURE);
 		}
 		if (B[n].weight < 0.0) {	/* Negative weight means invert sense of taper */
@@ -427,13 +429,13 @@ int init_blend_job (FILE *fp, struct GRD_HEADER *h, struct GRDBLEND_INFO **blend
 		GMT_close_grd (&B[n].G);
 
 		n++;
-		if (n == (int)n_alloc) {
-			n_alloc += GMT_SMALL_CHUNK;
+		if (n == (GMT_LONG)n_alloc) {
+			n_alloc <<= 1;
 			B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n_alloc, sizeof (struct GRDBLEND_INFO), GMT_program);
 		}
 	}
 
-	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, n, sizeof (struct GRDBLEND_INFO), GMT_program);
+	B = (struct GRDBLEND_INFO *) GMT_memory ((void *)B, (size_t)n, sizeof (struct GRDBLEND_INFO), GMT_program);
 	*blend = B;
 
 	return (n);
@@ -448,6 +450,7 @@ void sync_input_rows (int row, struct GRDBLEND_INFO *B, int n_blend, double half
 			if (B[k].open) {
 				GMT_close_grd (&B[k].G);	/* Done with this file */
 				B[k].open = FALSE;
+				GMT_free ((void *)B[k].z);
 			}
 			continue;
 		}
@@ -492,7 +495,7 @@ void decode_R (char *string, double *w, double *e, double *s, double *n) {
 void *New_Grdblend_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct GRDBLEND_CTRL *C;
 	
-	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, 1, sizeof (struct GRDBLEND_CTRL), "New_Grdblend_Ctrl");
+	C = (struct GRDBLEND_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct GRDBLEND_CTRL), "New_Grdblend_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
