@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: project.c,v 1.49 2007/05/07 01:42:18 pwessel Exp $
+ *	$Id: project.c,v 1.57 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -97,16 +97,18 @@ struct PROJECT_DATA {
 
 int main (int argc, char **argv)
 {
-	int	i, j, k, n_outputs, n_used, n_total_read, n_alloc = GMT_CHUNK;
+	GMT_LONG	i, n_used, n_total_read, n_total_used = 0, n_alloc = GMT_CHUNK;
+	
+	int	j, k, n_outputs;
 	int	n_files = 0, fno, n_args, n_fields, n_expected_fields;
-	int	n_total_used = 0, n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
+	int	n_z = 0, n_items, output_choice[PROJECT_N_FARGS];
 
 	double	xx, yy, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
 	double	theta, d_along, *in, *out = (double *)NULL;
 
 	double	a[3], b[3], x[3], xt[3], pole[3], center[3], e[9];
 
-	BOOLEAN	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, pure_ascii, skip;
+	BOOLEAN	dateline = FALSE, error = FALSE, find_new_point = FALSE, first = TRUE, z_first = TRUE, pure_ascii, skip;
 	BOOLEAN greenwich = FALSE, rads = TRUE, nofile = TRUE, done = FALSE, want_z_output = FALSE;
 
 	FILE *fp = NULL;
@@ -118,9 +120,6 @@ int main (int argc, char **argv)
 	struct PROJECT_CTRL *Ctrl;
 
 	int	compare_distances (const void *point_1, const void *point_2);
-	int	do_flat_earth ();
-	int	solve_right_spherical_triangle ();
-	int	sphere_azim_dist ();
 	double	oblique_setup (double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
 	void	oblique_transform (double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
 	void	make_euler_matrix (double *p, double *e, double *theta, int rads);
@@ -265,7 +264,7 @@ int main (int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_give_synopsis_and_exit) {
-		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordiantes\n\n", GMT_VERSION);
+		fprintf (stderr, "project %s - project data onto line or great circle, generate track, or translate coordinates\n\n", GMT_VERSION);
 		fprintf(stderr,"usage:	project [files] -C<ox>/<oy> [-A<azimuth>] [-D<d_or_g>] [-E<bx>/<by>]\n");
 		fprintf(stderr,"\t[-F<flags>] [-G<dist>] [%s] [-L[w][<l_min>/<l_max>]]\n", GMT_H_OPT);
 		fprintf(stderr,"\t[%s] [-N] [-Q] [-S] [-T<px>/<py>] [-V] [-W<w_min>/<w_max>]\n", GMT_M_OPT);
@@ -501,7 +500,7 @@ int main (int argc, char **argv)
 			n_used++;
 			d_along = Ctrl->L.min + n_used * Ctrl->G.inc;
 			if (n_used == (n_alloc-1)) {
-				n_alloc += GMT_CHUNK;
+				n_alloc <<= 1;
 				p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 			}
 		}
@@ -596,11 +595,6 @@ int main (int argc, char **argv)
 			}
 
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
-			n_z = n_expected_fields - 2;
-			if (n_z == 0 && want_z_output) {
-				fprintf (stderr, "%s: No data columns, cannot use z flag in -F\n", GMT_program);
-				exit (EXIT_FAILURE);
-			}
 			n_used = 0;
 
 			while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -609,10 +603,18 @@ int main (int argc, char **argv)
 					GMT_write_segmentheader (GMT_stdout, n_expected_fields);
 					n_fields = GMT_input (fp, &n_expected_fields, &in);
 				}
+				if (z_first) {
+					n_z = n_expected_fields - 2;
+					if (n_z == 0 && want_z_output) {
+						fprintf (stderr, "%s: No data columns, cannot use z flag in -F\n", GMT_program);
+						exit (EXIT_FAILURE);
+					}
+					z_first = FALSE;
+				}
 
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
@@ -653,7 +655,7 @@ int main (int argc, char **argv)
 					}
 					n_used++;
 					if (n_used == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						p_data = (struct PROJECT_DATA *) GMT_memory ((void *)p_data, (size_t)n_alloc, sizeof (struct PROJECT_DATA), GMT_program);
 					}
 
@@ -724,10 +726,8 @@ int main (int argc, char **argv)
 				if (n_z && pure_ascii) {
 					for (i = 0; i < n_used; i++) {
 						for (j = 0; j < n_outputs; j++) {
-							if (output_choice[j] == -1) {	/* Output all z columns as one string */
+							if (output_choice[j] == -1)	/* Output all z columns as one string */
 								fprintf (GMT_stdout, "%s", p_data[i].t);
-								GMT_free ((void *)p_data[i].t);
-							}
 							else
 								fprintf (GMT_stdout, gmtdefs.d_format, p_data[i].a[output_choice[j]]);
 							(j == (n_outputs - 1)) ? fprintf (GMT_stdout, "\n") : fprintf (GMT_stdout, "\t");
@@ -739,7 +739,6 @@ int main (int argc, char **argv)
 						for (j = k = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1) {	/* Copy over all z columns */
 								memcpy ((void *)&out[k], (void *)p_data[i].z, (size_t)(n_z * sizeof (double)));
-								GMT_free ((void *)p_data[i].z);
 								k += n_z;
 							}
 							else
@@ -758,8 +757,12 @@ int main (int argc, char **argv)
 		}
 	}
 
-	if (gmtdefs.verbose) fprintf(stderr, "%s: %d read, %d used\n", GMT_program, n_total_read, n_total_used);
+	if (gmtdefs.verbose) fprintf(stderr, "%s: %ld read, %ld used\n", GMT_program, n_total_read, n_total_used);
 
+	for (i = 0; i < n_alloc; i++) {
+		if (p_data[i].t) GMT_free ((void *)p_data[i].t);
+		if (p_data[i].z) GMT_free ((void *)p_data[i].z);
+	}
 	GMT_free ((void *)p_data);
 	GMT_free ((void *)out);
 
@@ -1000,7 +1003,7 @@ void copy_text_from_col3 (char *line, char *z_cols)
 void *New_Project_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PROJECT_CTRL *C;
 	
-	C = (struct PROJECT_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PROJECT_CTRL), "New_Project_Ctrl");
+	C = (struct PROJECT_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PROJECT_CTRL), "New_Project_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	
