@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psrose.c,v 1.59 2007/09/30 04:16:49 guru Exp $
+ *	$Id: psrose.c,v 1.66 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -52,6 +52,9 @@ struct PSROSE_CTRL {	/* All control options for this program (except common args
 		BOOLEAN active;
 		double azimuth, elevation;
 	} E;
+	struct F {	/* -F */
+		BOOLEAN active;
+	} F;
 	struct G {	/* -G<fill> */
 		BOOLEAN active;
 		struct GMT_FILL fill;
@@ -100,8 +103,10 @@ int main (int argc, char **argv)
 	char text[BUFSIZ], *file = CNULL, format[BUFSIZ], *choice[2] = {"OFF", "ON"};
 	char txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT], txt_d[GMT_LONG_TEXT];
 
-	int i, n_bins, bin, n_annot, n = 0, n_alloc = GMT_CHUNK;
+	int n_bins, bin, n_annot;
 	int status, n_alpha, n_modes, n_fields, n_expected_fields;
+	
+	GMT_LONG i, n = 0, n_alloc = GMT_CHUNK;
 
 	double max = 0.0, radius, az, x_origin, y_origin, tmp, one_or_two = 1.0, s, c, half_bin_width;
 	double angle1, angle2, angle, x, y, mean_theta, mean_radius, xr = 0.0, yr = 0.0;
@@ -173,6 +178,9 @@ int main (int argc, char **argv)
 					Ctrl->E.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
 					break;
+				case 'F':		/* Disable scalebar plotting */
+					Ctrl->F.active = TRUE;
+					break;
 				case 'G':		/* Set Gray shade */
 					Ctrl->G.active = TRUE;
 					if (GMT_getfill (&argv[i][2], &Ctrl->G.fill)) {
@@ -395,7 +403,7 @@ int main (int argc, char **argv)
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
-			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n);
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields,  n_expected_fields, n);
 			exit (EXIT_FAILURE);
 		}
 
@@ -424,7 +432,7 @@ int main (int argc, char **argv)
 
 		n++;
 		if (n == n_alloc) {	/* Get more memory */
-			n_alloc += GMT_CHUNK;
+			n_alloc <<= 1;
 			azimuth = (double *) GMT_memory ((void *)azimuth, (size_t)n_alloc, sizeof (double), GMT_program);
 			length = (double *) GMT_memory ((void *)length, (size_t)n_alloc, sizeof (double), GMT_program);
 		}
@@ -467,7 +475,7 @@ int main (int argc, char **argv)
 
 	if (Ctrl->I.active || gmtdefs.verbose) {
 		if (file) strcpy (text, file); else strcpy (text, "<stdin>");
-		sprintf (format, "%%s: Info for %%s: n = %%d rmax = %s mean r/az = (%s/%s) totlength = %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "%%s: Info for %%s: n = %%ld rmax = %s mean r/az = (%s/%s) totlength = %s\n", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		fprintf (stderr, format, GMT_program, text, n, max, mean_radius, mean_theta, total);
 		if (Ctrl->I.active) {
 			GMT_free ((void *)sum);
@@ -582,10 +590,11 @@ int main (int argc, char **argv)
 			n_modes = 0;
 			n_alloc = 1;
 			while (GMT_fgets (text, BUFSIZ, fpm)) {
+				if (GMT_is_a_blank_line (text)) continue;	/* Skip blank lines or # comments */
 				sscanf (text, "%lf %lf", &mode_direction[n_modes], &mode_length[n_modes]);
 				n_modes++;
 				if (n_modes == n_alloc) {
-					n_alloc += GMT_SMALL_CHUNK;
+					n_alloc <<= 1;
 					mode_direction = (double *) GMT_memory ((void *)mode_direction, (size_t)n_alloc, sizeof (double), "psrose");
 					mode_length = (double *) GMT_memory ((void *)mode_length, (size_t)n_alloc, sizeof (double), "psrose");
 				}
@@ -649,9 +658,9 @@ int main (int argc, char **argv)
 					sprintf (Ctrl->L.n, "0");
 				}
 				else {
-					sprintf (Ctrl->L.w, "90%cW", gmtdefs.encoding.code[which]);
-					sprintf (Ctrl->L.e, "90%cE", gmtdefs.encoding.code[which]);
-					sprintf (Ctrl->L.n, "0%c", gmtdefs.encoding.code[which]);
+					sprintf (Ctrl->L.w, "90%cW", (int)gmtdefs.encoding.code[which]);
+					sprintf (Ctrl->L.e, "90%cE", (int)gmtdefs.encoding.code[which]);
+					sprintf (Ctrl->L.n, "0%c",   (int)gmtdefs.encoding.code[which]);
 				}
 			}
 			y = -(3.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * asize);
@@ -670,21 +679,23 @@ int main (int argc, char **argv)
 		}
 		else {
 			GMT_text3D (0.0, -off - 2.0 * gmtdefs.annot_offset[0], project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.s, 0.0, 10, 0);
-			plot3d (off, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 2);
-			plot3d (off, -off, 3);
-			plot3d (off, gmtdefs.tick_length - off, 2);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 3);
-			plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, gmtdefs.tick_length - off, -2);
+			if (!Ctrl->F.active) {	/* Draw scale bar */
+				plot3d (off, -off, 3);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 2);
+				plot3d (off, -off, 3);
+				plot3d (off, gmtdefs.tick_length - off, 2);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -off, 3);
+				plot3d ((max_radius - frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, gmtdefs.tick_length - off, -2);
+				if (frame_info.axis[0].label[0]) {
+					strcat (format, " %s");
+					sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval, frame_info.axis[0].label);
+				}
+				else
+					sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
+				GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
+			}
 			y = -(off + 5.0 * gmtdefs.annot_offset[0] + GMT_font[gmtdefs.annot_font[0]].height * lsize + GMT_font[gmtdefs.label_font].height * lsize);
 			if (frame_info.axis[1].label[0]) GMT_text3D (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.axis[1].label, 0.0, 10, 0);
-			if (frame_info.axis[0].label[0]) {
-				strcat (format, " %s");
-				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval, frame_info.axis[0].label);
-			}
-			else
-				sprintf (text, format, frame_info.axis[0].item[GMT_GRID_UPPER].interval);
-			GMT_text3D ((max_radius - 0.5 * frame_info.axis[0].item[GMT_GRID_UPPER].interval) * Ctrl->S.scale, -(off + gmtdefs.annot_offset[0]), project_info.z_level, gmtdefs.annot_font_size[0], gmtdefs.annot_font[0], text, 0.0, 10, 0);
 		}
 		GMT_text3D (off + 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.e, 0.0, 5, 0);
 		GMT_text3D (-off - 2.0 * gmtdefs.annot_offset[0], 0.0, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, Ctrl->L.w, 0.0, 7, 0);
@@ -701,6 +712,8 @@ int main (int argc, char **argv)
 	GMT_free ((void *)yy);
 	GMT_free ((void *)azimuth);
 	GMT_free ((void *)length);
+	GMT_free ((void *)mode_length);
+	GMT_free ((void *)mode_direction);
 
 	Free_Psrose_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -716,8 +729,8 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 		double *xx, *yy;
 
 		n = GMT_get_arc (x, y, r, a1, a2, &xx, &yy);
-		GMT_2D_to_3D (xx, yy, project_info.z_level, n);
-		ps_line (xx, yy, n, status, FALSE, TRUE);
+		GMT_2D_to_3D (xx, yy, project_info.z_level, (GMT_LONG)n);
+		ps_line (xx, yy, (GMT_LONG)n, status, FALSE, TRUE);
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
@@ -734,7 +747,7 @@ void plot3d (double x, double y, int pen)
 void *New_Psrose_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSROSE_CTRL *C;
 	
-	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
+	C = (struct PSROSE_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSROSE_CTRL), "New_Psrose_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->E.azimuth = 180.0;
