@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxy.c,v 1.132 2007/09/30 04:16:49 guru Exp $
+ *	$Id: psxy.c,v 1.145 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -76,14 +76,16 @@ struct PSXY_CTRL {
 int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, read_symbol = FALSE, penset_OK = TRUE, outline_active;
-	BOOLEAN	default_outline = FALSE, done, get_rgb = FALSE, default_polygon = FALSE, resample, bar_clip;
+	BOOLEAN	default_outline = FALSE, done, get_rgb = FALSE, resample, bar_clip;
 	BOOLEAN old_GMT_world_map, error_x = FALSE, error_y = FALSE, def_err_xy = FALSE, clip_set = FALSE, fill_active;
 
 	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT];
 
-	int i, j, n, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
-	int n_alloc = GMT_CHUNK, plot_n, n_args, n_fields, n_cols_start = 2, this_outline, change;
-	int n_required, n_total_read = 0, n_expected = 0, reset_cap = 0, error_cols[3] = {1,4,5};
+	GMT_LONG i, n, n_alloc = GMT_CHUNK, n_total_read = 0, plot_n;
+	
+	int j, j0, n_files = 0, fno, xy_errors[2], two, three, four, error_type[2] = {0,0};
+	int n_args, n_fields, n_cols_start = 2, this_outline, change;
+	int n_required, n_expected = 0, reset_cap = 0, error_cols[3] = {1,4,5};
 
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, s, c, step;
 	double plot_x, plot_y, x_1, x_2, y_1, y_2, x0, y0, *xx, *yy, *xp, *yp;
@@ -98,10 +100,10 @@ int main (int argc, char **argv)
 	struct GMT_SYMBOL S;
 	struct PSXY_CTRL *Ctrl;
 
-	void plot_x_errorbar (double x, double y, double delta_x, double error_width2, int line);
-	void plot_y_errorbar (double x, double y, double delta_y, double error_width2, int line);
-	void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], int line, int notched);
-	void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], int line, int notched);
+	void plot_x_errorbar (double x, double y, double delta_x, double error_width2, GMT_LONG line);
+	void plot_y_errorbar (double x, double y, double delta_y, double error_width2, GMT_LONG line);
+	void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int notched);
+	void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int notched);
 	void *New_Psxy_Ctrl (), Free_Psxy_Ctrl (struct PSXY_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -172,13 +174,13 @@ int main (int argc, char **argv)
 					Ctrl->C.active = TRUE;
 					break;
 				case 'D':
-					if (sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b) != 2) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give x and y offsets\n", GMT_program);
+					if ((j = sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b)) >= 1) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give x [and y] offsets\n", GMT_program);
 						error++;
 					}
 					else {
 						Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
-						Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
+						Ctrl->D.dy = (j == 2) ? GMT_convert_units (txt_b, GMT_INCH) : Ctrl->D.dx;
 						Ctrl->D.active = TRUE;
 					}
 					break;
@@ -278,7 +280,7 @@ int main (int argc, char **argv)
 		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on z-value in 3rd column (requires -S)\n");
 		fprintf (stderr, "\t   Without -S, psxy excepts -M lines/polygons and looks for -Z<val> options in each multiheader\n");
 		fprintf (stderr, "\t   Then, color is applied to polygon fill (-L) or polygon pen\n");
-		fprintf (stderr, "\t-D Offset symbol positions by <dx>/<dy> [no offset]. Requires -S.\n");
+		fprintf (stderr, "\t-D Offset symbol or line positions by <dx>/<dy> [no offset].\n");
 		fprintf (stderr, "\t-E means draw error bars for x, y, or both.  Add cap-width [%g]. Append pen attributes;\n", Ctrl->E.size);
 		fprintf (stderr, "\t   leading + applies cpt color (-C) to both symbol fill and error pen, - applies it to pen only.\n");
 		fprintf (stderr, "\t   If X or Y is used instead a box-and-whisker diagram is drawn instead, using data from 4\n");
@@ -384,10 +386,6 @@ int main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have symbol information\n", GMT_program);
 		error++;
 	}
-	if (S.symbol == 0 && Ctrl->D.active) {
-		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -D option only applies to symbols, not lines\n", GMT_program);
-		error++;
-	}
 	if (Ctrl->E.active && Ctrl->E.mode && !Ctrl->C.active) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  -E option +|-<pen> requires the -C option\n", GMT_program);
 		error++;
@@ -445,7 +443,6 @@ int main (int argc, char **argv)
 	current_pen = default_pen = Ctrl->W.pen;
 	current_fill = default_fill = Ctrl->G.fill;
 	default_outline = Ctrl->W.active;
-	default_polygon = polygon;
 
 	Ctrl->E.size *= 0.5;	/* Since we draw half-way in either direction */
 
@@ -528,7 +525,11 @@ int main (int argc, char **argv)
 	if (S.symbol == GMT_SYMBOL_BARY && !S.base_set && project_info.xyz_projection[GMT_Y] == GMT_LOG10) S.base = south;	/* Default to south level for vertical log10 bars */
 	if (S.symbol == GMT_SYMBOL_BARX && !S.base_set && project_info.xyz_projection[GMT_X] == GMT_LOG10) S.base = west;	/* Default to west level for horizontal log10 bars */
 	bar_clip = ((S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && !Ctrl->N.active) ? TRUE : FALSE;		/* Bars get clipped instead of skipped */
-
+	if ((S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR) && S.v_just == 3) {
+		/* Reading 2nd coordinate so must set column types */
+		GMT_io.in_col_type[two] = GMT_io.in_col_type[GMT_X];
+		GMT_io.in_col_type[three] = GMT_io.in_col_type[GMT_Y];
+	}
 	fill_active = Ctrl->G.active;	/* Make copies because we will change the values */
 	outline_active =  Ctrl->W.active;
 	if (S.symbol > 0 && !outline_active && !fill_active && !get_rgb) outline_active = TRUE;		/* If no fill nor outline for symbols then turn outline on */
@@ -537,6 +538,7 @@ int main (int argc, char **argv)
 		xx = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	}
+	if (Ctrl->D.active) ps_transrotate (Ctrl->D.dx, Ctrl->D.dy, 0.0);	/* Shift plot a bit */
 	done = FALSE;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -583,15 +585,11 @@ int main (int argc, char **argv)
 				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
 				if (GMT_io.status & GMT_IO_MISMATCH && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
 	
-				if (S.read_size) {
-					S.size_x = in[two] * GMT_u2u[S.u][GMT_INCH];
-					S.size_x2 = 0.5 * S.size_x;
-				}
 				if (read_symbol) {
 					i = strlen (GMT_io.current_record) - 1;
 					GMT_io.current_record[i--] = 0;	/* Chop off the \n */
@@ -599,6 +597,10 @@ int main (int argc, char **argv)
 					(void ) GMT_parse_symbol_option (&GMT_io.current_record[i+1], &S, 0, FALSE);
 					n_required = n_cols_start + S.n_required;
 				}
+				if (S.read_size) {
+					S.size_x = in[two] * GMT_u2u[S.u][GMT_INCH];
+					S.size_x2 = 0.5 * S.size_x;
+				}
 
 				if (S.symbol == GMT_SYMBOL_BARY) GMT_geo_to_xy (west, S.base, &plot_x, &y0);	/* Zero y level for vertical bars */
 				if (S.symbol == GMT_SYMBOL_BARX) GMT_geo_to_xy (S.base, south, &x0, &plot_y);	/* Zero x level for horizontal bars */
@@ -640,8 +642,8 @@ int main (int argc, char **argv)
 				}
 				else if (S.symbol == GMT_SYMBOL_ROTATERECT) {
 					direction = in[two];
-					x_len = in[three+S.read_size];
-					y_len = in[four+S.read_size];
+					x_len = in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH];
+					y_len = in[four+S.read_size] * GMT_u2u[S.u][GMT_INCH];
 				}
 				else if (S.symbol == GMT_SYMBOL_PIE && S.convert_angles) {
 					dir1 = 90.0 - in[two+S.read_size];
@@ -652,23 +654,23 @@ int main (int argc, char **argv)
 					dir2 = in[three+S.read_size];
 				}
 				else if (S.symbol == GMT_SYMBOL_RECT) {
-					x_len = in[two+S.read_size];
-					y_len = in[three+S.read_size];
+					x_len = in[two+S.read_size] * GMT_u2u[S.u][GMT_INCH];
+					y_len = in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH];
 				}
 		
 				/* Skip zero-size symbols */
 		
 				if (!(S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && S.symbol < GMT_SYMBOL_ELLIPSE && S.size_x <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 		
 				if (S.read_vector && S.v_just < 3 && length <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Vector length <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Vector length <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (S.symbol == GMT_SYMBOL_ELLIPSE && (major <= 0.0 || minor <= 0.0)) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Ellipse axes <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Ellipse axes <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (bar_clip) {
@@ -692,17 +694,13 @@ int main (int argc, char **argv)
 
 				GMT_geo_to_xy (in[0], in[1], &plot_x, &plot_y);
 				if (GMT_is_dnan (plot_x)) {
-					fprintf (stderr, "%s: Data point with x = NaN near line %d\n", GMT_program, n_total_read);
+					fprintf (stderr, "%s: Data point with x = NaN near line %ld\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (GMT_is_dnan (plot_y)) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Data point with y = NaN near line %d\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Data point with y = NaN near line %ld\n", GMT_program, n_total_read);
 					continue;
 				}
-				if (Ctrl->D.active) {
-					plot_x += Ctrl->D.dx;
-					plot_y += Ctrl->D.dy;
-				}
 				if (Ctrl->W.mode) memcpy ((void *)Ctrl->W.pen.rgb, (void *)current_fill.rgb, 3*sizeof (int));
 				if (Ctrl->E.active) {
 					if (Ctrl->E.mode) memcpy ((void *)Ctrl->E.pen.rgb, (void *)current_fill.rgb, 3*sizeof (int));
@@ -832,9 +830,9 @@ int main (int argc, char **argv)
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
 							direction = 90.0 - direction;
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[2], in[3], &x2, &y2);
+							GMT_geo_to_xy (in[two], in[three], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -859,9 +857,9 @@ int main (int argc, char **argv)
 						else if (S.convert_angles == 1)	/* Cartesian azimuth */
 							direction = 90.0 - direction;
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[2], in[3], &x2, &y2);
+							GMT_geo_to_xy (in[two], in[three], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -940,14 +938,14 @@ int main (int argc, char **argv)
 				n = 0;
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
 					xx[n] = in[0];	yy[n] = in[1];
 					n++;
 					if (n == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
 						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
@@ -958,7 +956,7 @@ int main (int argc, char **argv)
 					yy[n] = yy[0];
 					n++;
 					if (n == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						xx = (double *) GMT_memory ((void *)xx, (size_t)n_alloc, sizeof (double), GMT_program);
 						yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 					}
@@ -972,13 +970,12 @@ int main (int argc, char **argv)
 				
 				if (resample) n = GMT_fix_up_path (&xx, &yy, n, step, Ctrl->A.mode);	/* Resample if spacing is too coarse */
 
-				if (polygon) {	/* Want a filled polygon */
-					GMT_fill_polygon (xx, yy, 0.0, n, &current_fill, outline_active);
-				}
+				if (polygon)	/* Want a filled polygon */
+					GMT_fill_polygon (xx, yy, 0.0, n, &current_fill, outline_active);	/* NO outline since path may have been clipped */
 				else {		/* Want a line of some sort */
 					if ((plot_n = GMT_geo_to_xy_line (xx, yy, n)) == 0) continue;		/* Nothing there there */
 					if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {	/* Labelled lines are dealt with by the contour machinery */
-						int start = 0, stop;
+						GMT_LONG start = 0, stop;
 						S.G.line_pen = current_pen;
 						/* Since the line might exit and enter the region we must isolate the segments before calling GMT_hold_contour */
 						while (start < plot_n) {	/* More segments to register */
@@ -1008,7 +1005,12 @@ int main (int argc, char **argv)
 
 	if (reset_cap == 1) ps_setlinecap (gmtdefs.ps_line_cap);	/* Reset linecap to default */
 
-	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) GMT_contlabel_plot (&S.G);
+	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
+		GMT_contlabel_plot (&S.G);
+		GMT_contlabel_free (&S.G);
+	}
+
+	if (Ctrl->D.active) ps_transrotate (-Ctrl->D.dx, -Ctrl->D.dy, 0.0);	/* Reset shift */
 
 	if (clip_set) GMT_map_clip_off ();
 
@@ -1018,7 +1020,7 @@ int main (int argc, char **argv)
 
 	GMT_plotend ();
 
-	if (S.symbol == 0) {
+	if (S.symbol <= 0) {
 		GMT_free ((void *)xx);
 		GMT_free ((void *)yy);
 	}
@@ -1030,17 +1032,17 @@ int main (int argc, char **argv)
 	exit (EXIT_SUCCESS);
 }
 
-void plot_x_errorbar (double x, double y, double delta_x, double error_width2, int line) {
+void plot_x_errorbar (double x, double y, double delta_x, double error_width2, GMT_LONG line) {
 	double x_1, x_2, y_1, y_2;
 
 	GMT_geo_to_xy (x - delta_x, y, &x_1, &y_1);
 	GMT_geo_to_xy (x + delta_x, y, &x_2, &y_2);
 	if (GMT_is_dnan (x_1)) {
-		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %d. Set to x_min\n", GMT_program, line);
+		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %ld. Set to x_min\n", GMT_program, line);
 		x_1 = project_info.xmin;
 	}
 	if (GMT_is_dnan (x_2)) {
-		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %d. Set to x_max\n", GMT_program, line);
+		fprintf (stderr, "%s: Warning: X error bar exceeded domain near line %ld. Set to x_max\n", GMT_program, line);
 		x_2 = project_info.xmax;
 	}
 	ps_segment (x_1, y_1, x_2, y_2);
@@ -1050,17 +1052,17 @@ void plot_x_errorbar (double x, double y, double delta_x, double error_width2, i
 	}
 }
 
-void plot_y_errorbar (double x, double y, double delta_y, double error_width2, int line) {
+void plot_y_errorbar (double x, double y, double delta_y, double error_width2, GMT_LONG line) {
 	double x_1, x_2, y_1, y_2;
 
 	GMT_geo_to_xy (x, y - delta_y, &x_1, &y_1);
 	GMT_geo_to_xy (x, y + delta_y, &x_2, &y_2);
 	if (GMT_is_dnan (y_1)) {
-		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %d. Set to y_min\n", GMT_program, line);
+		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %ld. Set to y_min\n", GMT_program, line);
 		y_1 = project_info.ymin;
 	}
 	if (GMT_is_dnan (y_2)) {
-		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %d. Set to y_max\n", GMT_program, line);
+		fprintf (stderr, "%s: Warning: Y error bar exceeded domain near line %ld. Set to y_max\n", GMT_program, line);
 		y_2 = project_info.ymax;
 	}
 	ps_segment (x_1, y_1, x_2, y_2);
@@ -1070,7 +1072,7 @@ void plot_y_errorbar (double x, double y, double delta_y, double error_width2, i
 	}
 }
 
-void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], int line, int kind) {
+void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int kind) {
 	int i;
 	static int q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
@@ -1078,7 +1080,7 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2,
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (hinge[i], y, &xx[i], &yy[i]);
 		if (GMT_is_dnan (xx[i])) {
-			fprintf (stderr, "%s: Warning: X %d %% hinge exceeded domain near line %d\n", GMT_program, q[i], line);
+			fprintf (stderr, "%s: Warning: X %d %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
 			xx[i] = (i <2 ) ? project_info.xmin :  project_info.xmax;
 		}
 	}
@@ -1103,7 +1105,7 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2,
 		yp[5] = yp[6] = yp[8] = yp[9] = yy[2];
 		yp[2] = yy[0] - 0.5 * error_width2;
 		yp[7] = yy[0] + 0.5 * error_width2;
-		ps_polygon (xp, yp, 10, rgb, TRUE);
+		ps_patch (xp, yp, (GMT_LONG)10, rgb, TRUE);
 		ps_segment (x, yp[7], x, yp[2]);			/* Median line */
 	}
 	else {
@@ -1112,7 +1114,7 @@ void plot_x_whiskerbar (double x, double y, double hinge[], double error_width2,
 	}
 }
 
-void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], int line, int kind) {
+void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2, int rgb[], GMT_LONG line, int kind) {
 	int i;
 	static int q[4] = {0, 25, 75, 100};
 	double xx[4], yy[4];
@@ -1120,7 +1122,7 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2,
 	for (i = 0; i < 4; i++) {	/* for 0, 25, 75, 100% hinges */
 		GMT_geo_to_xy (x, hinge[i], &xx[i], &yy[i]);
 		if (GMT_is_dnan (yy[i])) {
-			fprintf (stderr, "%s: Warning: Y %d %% hinge exceeded domain near line %d\n", GMT_program, q[i], line);
+			fprintf (stderr, "%s: Warning: Y %d %% hinge exceeded domain near line %ld\n", GMT_program, q[i], line);
 			yy[i] = (i <2 ) ? project_info.ymin :  project_info.ymax;
 		}
 	}
@@ -1145,7 +1147,7 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2,
 		yp[2] = yp[7] = y;
 		yp[4] = yp[5] = yy[2];
 		yp[3] = yp[6] = ((p = (y + s)) > yp[4]) ? yp[4] : p;
-		ps_polygon (xp, yp, 10, rgb, TRUE);
+		ps_patch (xp, yp, (GMT_LONG)10, rgb, TRUE);
 		ps_segment (xp[7], y, xp[2], y);			/* Median line */
 	}
 	else {
@@ -1157,7 +1159,7 @@ void plot_y_whiskerbar (double x, double y, double hinge[], double error_width2,
 void *New_Psxy_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSXY_CTRL *C;
 	
-	C = (struct PSXY_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSXY_CTRL), "New_Psxy_Ctrl");
+	C = (struct PSXY_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSXY_CTRL), "New_Psxy_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
