@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: psxyz.c,v 1.119 2007/09/30 04:16:49 guru Exp $
+ *	$Id: psxyz.c,v 1.130 2008/05/01 03:00:39 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,10 @@ struct PSXYZ_CTRL {
 		BOOLEAN active;
 		char *file;
 	} C;
+	struct D {	/* -D<dx>/<dy>[/<dz>] */
+		BOOLEAN active;
+		double dx, dy, dz;
+	} D;
 	struct E {	/* -E<azimuth/elevation> */
 		BOOLEAN active;
 		double azimuth, elevation;
@@ -94,15 +98,17 @@ int main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, done, penset_OK = TRUE;
 	BOOLEAN	get_rgb = FALSE, read_symbol, clip_set = FALSE, fill_active, bar_clip;
-	BOOLEAN default_outline = FALSE, default_polygon = FALSE, outline_active;
+	BOOLEAN default_outline = FALSE, outline_active;
 
-	char line[BUFSIZ];
+	char line[BUFSIZ], txt_a[GMT_LONG_TEXT], txt_b[GMT_LONG_TEXT], txt_c[GMT_LONG_TEXT];
 
-	int 	i, j, n, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
-	int	n_alloc = GMT_CHUNK, n_args, three, four, five, n_required = 3;
-	int	n_total_read = 0, n_expected = 0, this_outline, change;
+	GMT_LONG	i, n, n_alloc = GMT_CHUNK, n_total_read = 0;
+	
+	int 	j, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
+	int	n_args, three, four, five, n_required = 3;
+	int	n_expected = 0, this_outline, change;
 
-	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, dim[3];
+	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, dim[3], DX, DY;
 	double lux[3], tmp, *in, x_1, x_2, y_1, y_2, xxx, yyy, font_size;
 	double x2, y2, v_w, h_l, h_w, dx, dy, xx[2], yy[2], s, c, size, *xp, *yp;
 
@@ -119,7 +125,7 @@ int main (int argc, char **argv)
 	void cube3D (double x, double y, double z, double x_size, double y_size, int *rgb, int outline);
 	void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
 	void bary3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline);
-	void sort_on_distance (struct PSXYZ_DATA1 *data, int n);
+	void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n);
 	void *New_Psxyz_Ctrl (), Free_Psxyz_Ctrl (struct PSXYZ_CTRL *C);
 	
 	argc = GMT_begin (argc, argv);
@@ -180,6 +186,18 @@ int main (int argc, char **argv)
 					Ctrl->C.file = strdup (&argv[i][2]);
 					Ctrl->C.active = TRUE;
 					break;
+				case 'D':
+					if ((n = sscanf (&argv[i][2], "%[^/]/%[^/]/%s", txt_a, txt_b, txt_c)) < 2) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -D option: Give x and y [and z] offsets\n", GMT_program);
+						error++;
+					}
+					else {
+						Ctrl->D.dx = GMT_convert_units (txt_a, GMT_INCH);
+						Ctrl->D.dy = GMT_convert_units (txt_b, GMT_INCH);
+						if (n == 3) Ctrl->D.dz = GMT_convert_units (txt_c, GMT_INCH);
+						Ctrl->D.active = TRUE;
+					}
+					break;
 				case 'E':
 					Ctrl->E.active = TRUE;
 					sscanf (&argv[i][2], "%lf/%lf", &Ctrl->E.azimuth, &Ctrl->E.elevation);
@@ -237,7 +255,7 @@ int main (int argc, char **argv)
 	if (argc == 1 || GMT_give_synopsis_and_exit) {	/* Display usage */
 		fprintf (stderr,"psxyz %s - Plot lines, polygons, and symbols in 3-D\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: psxyz <xyzfiles> %s %s [%s]\n", GMT_J_OPT, GMT_Rgeoz_OPT, GMT_B_OPT);
-		fprintf(stderr, "\t[%s] [-C<cpt>] [-E<az/el>] [-G<fill>] [%s] [-K] [-L] [%s] [-N] [-O]\n", GMT_Jz_OPT, GMT_Ho_OPT, GMT_Mo_OPT);
+		fprintf(stderr, "\t[%s] [-C<cpt>] [-D<dx>/<dy>[/<dz>]] [-E<az/el>] [-G<fill>] [%s] [-K] [-L] [%s] [-N] [-O]\n", GMT_Jz_OPT, GMT_Ho_OPT, GMT_Mo_OPT);
 		fprintf(stderr, "\t[-P] [-Q] [-S<symbol><size>[/size_y]] [%s] [-V] [-W[+|-][<pen>]] [%s]\n", GMT_U_OPT, GMT_X_OPT);
 		fprintf(stderr, "\t[%s] [%s] [%s] [%s] [%s]\n", GMT_Y_OPT, GMT_c_OPT, GMT_t_OPT, GMT_bi_OPT, GMT_f_OPT);
 
@@ -250,6 +268,7 @@ int main (int argc, char **argv)
 		GMT_explain_option ('b');
 		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on t-value in 4th column (requires -S)\n");
 		fprintf (stderr, "\t   Without -S, psxyz expects -M lines/polygons and looks for -Z<tval> strings in each multiheader\n");
+		fprintf (stderr, "\t-D Offset symbol or line positions by <dx>/<dy>[/<dy>] [no offset].\n");
 		fprintf (stderr, "\t-E set user viewpoint azimuth and elevation [180/90].\n");
 		GMT_fill_syntax ('G', "Specify color or pattern [Default is no fill].");
 		fprintf (stderr, "\t   If -G is specified but not -S, psxyz draws a filled polygon.\n");
@@ -370,7 +389,6 @@ int main (int argc, char **argv)
 	default_pen = current_pen = Ctrl->W.pen;
 	default_fill = current_fill = Ctrl->G.fill;
 	default_outline = Ctrl->W.active;
-	default_polygon = polygon;
 
 	if (get_rgb) n_cols_start++;
 
@@ -442,10 +460,20 @@ int main (int argc, char **argv)
 		ps_setfont (S.font_no);		/* Set the required font */
 	}
 	if (S.convert_angles && GMT_IS_MAPPING) S.convert_angles = 2;
+	if ((S.symbol == GMT_SYMBOL_VECTOR || S.symbol == GMT_SYMBOL_VECTOR) && S.v_just == 3) {
+		/* Reading 2nd coordinate so must set column types */
+		GMT_io.in_col_type[three] = GMT_io.in_col_type[GMT_X];
+		GMT_io.in_col_type[four] = GMT_io.in_col_type[GMT_Y];
+	}
 	fill_active = Ctrl->G.active;	/* Make copies because we will change the values */
 	outline_active =  Ctrl->W.active;
 	if (S.symbol > 0 && !outline_active && !fill_active && !get_rgb) outline_active = TRUE;		/* If no fill nor outline for symbols then turn outline on */
-	bar_clip = ((S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && !Ctrl->N.active) ? TRUE : FALSE;		/* Bars get clipped instead of skipped */
+	bar_clip = ((S.symbol == GMT_SYMBOL_BARX || S.symbol == GMT_SYMBOL_BARY) && !Ctrl->N.active) ? TRUE : FALSE;
+			/* Bars get clipped instead of skipped */
+	if (Ctrl->D.active) {
+		GMT_xyz_to_xy (Ctrl->D.dx, Ctrl->D.dy, Ctrl->D.dz, &DX, &DY);
+		ps_transrotate (DX, DY, 0.0);	/* Shift plot a bit */
+	}
 
 	done = FALSE;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
@@ -497,17 +525,11 @@ int main (int argc, char **argv)
 				if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 	
 				if (GMT_io.status & GMT_IO_MISMATCH && !read_symbol) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 					exit (EXIT_FAILURE);
 				}
 				n_total_read++;
 
-				/* S.size_x = S.size_y = 0.0; */
-				if (S.read_size) {
-					S.size_x = in[three] * GMT_u2u[S.u][GMT_INCH];
-					S.size_y = (S.symbol == GMT_SYMBOL_COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
-				}
-
 				if (read_symbol) {
 					i = strlen (GMT_io.current_record) - 1;
 					GMT_io.current_record[i--] = 0;	/* Chop off the \n */
@@ -519,6 +541,10 @@ int main (int argc, char **argv)
 					GMT_parse_symbol_option (Ctrl->S.arg, &S, 1, FALSE);
 					n_required = n_cols_start + S.n_required;
 				} */
+				if (S.read_size) {
+					S.size_x = in[three] * GMT_u2u[S.u][GMT_INCH];
+					S.size_y = (S.symbol == GMT_SYMBOL_COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
+				}
 
 				data1[n].flag = 0;
 				data1[n].lon = in[0];
@@ -572,8 +598,8 @@ int main (int argc, char **argv)
 					}
 				}
 				else if (S.symbol == GMT_SYMBOL_RECT) {
-					data1[n].x_size = (float)(in[three+S.read_size]);	/* x-dim */
-					data1[n].y_size = (float)(in[four+S.read_size]);	/* y-dim */
+					data1[n].x_size = (float)(in[three+S.read_size] * GMT_u2u[S.u][GMT_INCH]);	/* x-dim */
+					data1[n].y_size = (float)(in[four+S.read_size] * GMT_u2u[S.u][GMT_INCH]);	/* y-dim */
 				}
 				else {
 					data1[n].x_size = (float)S.size_x;
@@ -584,15 +610,15 @@ int main (int argc, char **argv)
 				/* Skip zero-size symbols */
 		
 				if (S.symbol != GMT_SYMBOL_POINT && S.symbol < GMT_SYMBOL_ELLIPSE && S.size_x <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (S.read_vector && S.v_just < 3 && data1[n].y_size <= 0.0) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: vector length <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: vector length <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 				if (S.symbol == GMT_SYMBOL_ELLIPSE && (data1[n].y_size <= 0.0 || data1[n].z_size <= 0.0)) {
-					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: ellipse axes <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: ellipse axes <= 0.0 near line %ld (skipped)\n", GMT_program, n_total_read);
 					continue;
 				}
 
@@ -659,7 +685,7 @@ int main (int argc, char **argv)
 
 				n++;
 				if (n == n_alloc) {
-					n_alloc += GMT_CHUNK;
+					n_alloc <<= 1;
 					data1 = (struct PSXYZ_DATA1 *) GMT_memory ((void *)data1, (size_t)n_alloc, sizeof (struct PSXYZ_DATA1), GMT_program);
 				}
 				if (read_symbol) n_expected = BUFSIZ;
@@ -808,9 +834,9 @@ int main (int argc, char **argv)
 							data1[i].x_size = (float)(90.0 - data1[i].x_size);
 						}
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[3], in[4], &x2, &y2);
+							GMT_geo_to_xy (in[three], in[four], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -836,9 +862,9 @@ int main (int argc, char **argv)
 							data1[i].x_size = (float)(90.0 - data1[i].x_size);
 						}
 						if (S.v_just == 3) {
-							GMT_geo_to_xy (in[3], in[4], &x2, &y2);
+							GMT_geo_to_xy (in[three], in[four], &x2, &y2);
 							if (GMT_is_dnan (x2) || GMT_is_dnan (y2)) {
-								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %d. Skipped\n", GMT_program, n_total_read);
+								fprintf (stderr, "%s: Warning: Vector head coordinates contain NaNs near line %ld. Skipped\n", GMT_program, n_total_read);
 								continue;
 							}
 						}
@@ -897,6 +923,7 @@ int main (int argc, char **argv)
 						GMT_draw_custom_symbol (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, S.custom, &current_pen, &current_fill, data1[i].outline);
 						break;
 				}
+				if (data1[i].string) GMT_free ((void *)data1[i].string);
 			}
 			GMT_free ((void *)data1);
 		}
@@ -928,7 +955,7 @@ int main (int argc, char **argv)
 				n = 0;
 				while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
 					if (GMT_io.status & GMT_IO_MISMATCH) {
-						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected, n_total_read);
+						fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected, n_total_read);
 						exit (EXIT_FAILURE);
 					}
 
@@ -936,7 +963,7 @@ int main (int argc, char **argv)
 					data2[n].z = in[2];
 					n++;
 					if (n == n_alloc) {
-						n_alloc += GMT_CHUNK;
+						n_alloc <<= 1;
 						data2 = (struct PSXYZ_DATA2 *) GMT_memory ((void *)data2, (size_t)n_alloc, sizeof (struct PSXYZ_DATA2), GMT_program);
 					}
 					n_fields = GMT_input (fp, &n_expected, &in);
@@ -987,7 +1014,12 @@ int main (int argc, char **argv)
 
 	}
 
-	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) GMT_contlabel_plot (&S.G);
+	if (S.symbol == GMT_SYMBOL_QUOTED_LINE) {
+		GMT_contlabel_plot (&S.G);
+		GMT_contlabel_free (&S.G);
+	}
+
+	if (Ctrl->D.active) ps_transrotate (-DX, -DY, 0.0);	/* Shift plot a bit */
 
 	if (clip_set) GMT_map_clip_off ();
 
@@ -1042,7 +1074,7 @@ void column3D (double x, double y, double z, double base, double x_size, double
 		}
 		k = z_project.face[i] / 2;
 		for (j = 0; j < 4; j++) GMT_xyz_to_xy (xp[j], yp[j], zp[j], &plot_x[j], &plot_y[j]);
-		ps_patch (plot_x, plot_y, 4, &rgb[3*k], outline);
+		ps_patch (plot_x, plot_y, (GMT_LONG)4, &rgb[3*k], outline);
 	}
 }
 
@@ -1080,7 +1112,7 @@ void cube3D (double x, double y, double z, double x_size, double y_size, int *rg
 		}
 		k = z_project.face[i] / 2;
 		for (j = 0; j < 4; j++) GMT_xyz_to_xy (xp[j], yp[j], zp[j], &plot_x[j], &plot_y[j]);
-		ps_patch (plot_x, plot_y, 4, &rgb[3*k], outline);
+		ps_patch (plot_x, plot_y, (GMT_LONG)4, &rgb[3*k], outline);
 	}
 }
 
@@ -1093,7 +1125,7 @@ void bary3D (double x, double y, double z, double base, double size, struct GMT_
 	xp[0] = xp[3] = x - size;	xp[1] = xp[2] = x + size;
 	yp[0] = yp[1] = base;	yp[2] = yp[3] = y;
 	for (i = 0; i < 4; i++) GMT_xyz_to_xy (xp[i], yp[i], z, &plot_x[i], &plot_y[i]);
-	GMT_fill (plot_x, plot_y, 4, f, outline);
+	GMT_fill (plot_x, plot_y, (GMT_LONG)4, f, outline);
 }
 
 void barx3D (double x, double y, double z, double base, double size, struct GMT_FILL *f, int outline)
@@ -1105,13 +1137,13 @@ void barx3D (double x, double y, double z, double base, double size, struct GMT_
 	yp[0] = yp[3] = y - size;	yp[1] = yp[2] = y + size;
 	xp[0] = xp[1] = base;	xp[2] = xp[3] = x;
 	for (i = 0; i < 4; i++) GMT_xyz_to_xy (xp[i], yp[i], z, &plot_x[i], &plot_y[i]);
-	GMT_fill (plot_x, plot_y, 4, f, outline);
+	GMT_fill (plot_x, plot_y, (GMT_LONG)4, f, outline);
 }
 
-void sort_on_distance (struct PSXYZ_DATA1 *data, int n)
+void sort_on_distance (struct PSXYZ_DATA1 *data, GMT_LONG n)
 {
 	/* This function sorts the data array such that points farthest away are plotted first */
-	int i;
+	GMT_LONG i;
 
 	int dist_compare(const void *point_1, const void *point_2);
 	double dx, dy, x0, y0, x, y, dr, a, b, c;
@@ -1164,7 +1196,7 @@ int dist_compare (const void *point_1, const void *point_2)
 void *New_Psxyz_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct PSXYZ_CTRL *C;
 	
-	C = (struct PSXYZ_CTRL *) GMT_memory (VNULL, 1, sizeof (struct PSXYZ_CTRL), "New_Psxyz_Ctrl");
+	C = (struct PSXYZ_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct PSXYZ_CTRL), "New_Psxyz_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 		
