@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *	$Id: xyz2grd.c,v 1.72 2007/09/30 04:18:33 guru Exp $
+ *	$Id: xyz2grd.c,v 1.78 2008/03/24 08:58:32 guru Exp $
  *
- *	Copyright (c) 1991-2007 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2008 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -77,10 +77,13 @@ int main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, do_grid = FALSE, count = FALSE, average = TRUE;
 	BOOLEAN b_only = FALSE, done = FALSE, nofile = TRUE;
-
-	int i, gmt_ij, ii, jj, z = 2, nm = 0, n_read = 0, n_filled = 0, n_used = 0, fno, n_args, *flag = VNULL;
-	int ij = -1;	/* Will be incremented to 0 or set first time around */
-	int n_empty = 0, n_stuffed = 0, n_bad = 0, n_expected_fields, n_fields, n_confused = 0, n_files = 0, high_low = 0;
+	
+	int i, ii, jj , z = 2, fno, n_args, *flag = VNULL;
+	int n_expected_fields, n_fields, n_files = 0, high_low = 0;
+	
+	GMT_LONG gmt_ij, nm = 0, n_read = 0, n_filled = 0, n_used = 0;
+	GMT_LONG ij = -1;	/* Will be incremented to 0 or set first time around */
+	GMT_LONG n_empty = 0, n_stuffed = 0, n_bad = 0, n_confused = 0;
 
 	double *in, d_value, wesn[4];
 
@@ -276,7 +279,7 @@ int main (int argc, char **argv)
 			error++;
 		}
 	}
-	if (!Ctrl->S.active && !Ctrl->G.file) {
+	if (!Ctrl->G.active && !Ctrl->G.file) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR option -G:  Must specify output file\n", GMT_program);
 		error++;
 	}
@@ -417,7 +420,7 @@ int main (int argc, char **argv)
 			while ((io.read_item) (fp, &d_value)) {
 				ij++;
 				if (ij == io.n_expected) {
-					fprintf (stderr, "%s: More than %d records, only %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+					fprintf (stderr, "%s: More than %ld records, only %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 					exit (EXIT_FAILURE);
 				}
 				(io.get_gmt_ij) (&io, ij, &gmt_ij);
@@ -428,14 +431,14 @@ int main (int argc, char **argv)
 			}
 			ij++;
 			if (ij != io.n_expected) {	/* Input amount doesn't match expectations */
-				fprintf (stderr, "%s: Found %d records, but %d was expected (aborting)!\n", GMT_program, ij, io.n_expected);
+				fprintf (stderr, "%s: Found %ld records, but %ld was expected (aborting)!\n", GMT_program, ij, io.n_expected);
 				exit (EXIT_FAILURE);
 			}
 
 			GMT_check_z_io (&io, a);	/* This fills in missing periodic row or column */
 		}
 		else if (Ctrl->E.active) {	/* Read an ESRI Arc Interchange grid format in ASCII */
-			int n_left, j;
+			GMT_LONG n_left, j;
 			float value;
 			
 			grd.node_offset = 0;
@@ -454,12 +457,13 @@ int main (int argc, char **argv)
 				fprintf (stderr, "%s: Error decoding xll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
-			if (!strncmp (line, "xllcorner", 9)) grd.node_offset = 1;	/* Pixel grid */
+			if (!strncmp (line, "xllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
 			GMT_fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.y_min) != 1) {
 				fprintf (stderr, "%s: Error decoding yll record\n", GMT_program);
 				exit (EXIT_FAILURE);
 			}
+			if (!strncmp (line, "yllcorner", (size_t)9)) grd.node_offset = 1;	/* Pixel grid */
 			GMT_fgets (line, BUFSIZ, fp);
 			if (sscanf (line, "%*s %lf", &grd.x_inc) != 1) {
 				fprintf (stderr, "%s: Error decoding cellsize record\n", GMT_program);
@@ -493,7 +497,7 @@ int main (int argc, char **argv)
 				n_left--;
 			}
 			if (n_left) {
-				fprintf (stderr, "%s: Expected %d points, found only %d\n", GMT_program, nm, nm - n_left);
+				fprintf (stderr, "%s: Expected %ld points, found only %ld\n", GMT_program, nm, nm - n_left);
 				exit (EXIT_FAILURE);
 			}
 		}
@@ -502,7 +506,7 @@ int main (int argc, char **argv)
 			while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 				if (GMT_io.status & GMT_IO_MISMATCH) {
-					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n_read);
+					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %ld\n", GMT_program, n_fields, n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
 				n_read++;
@@ -518,7 +522,7 @@ int main (int argc, char **argv)
 				jj = GMT_y_to_j (in[1], grd.y_min, grd.y_inc, grd.xy_off, grd.ny);
 				if (jj == -1) jj++, n_confused++;
 				if (jj == grd.ny) jj--, n_confused++;
-				ij = jj * grd.nx + ii;
+				ij = GMT_IJ (jj, ii, grd.nx);
 				if (high_low) {	/* Always come here if looking for extreme values */
 					if (flag[ij]) {	/* Already assigned the first value */
 						if ((high_low == -1 && (in[z] < (double)a[ij])) || (high_low == +1 && (in[z] > (double)a[ij]))) a[ij] = (float)in[z];
@@ -540,7 +544,7 @@ int main (int argc, char **argv)
 
 	if (!(Ctrl->Z.active || Ctrl->E.active)) {	/* xyz data could have resulted in duplicates */
 		if (GMT_io.in_col_type[0] == GMT_IS_LON && GMT_360_RANGE (grd.x_max, grd.x_min) && !grd.node_offset) {	/* Make sure longitudes got replicated */
-			int j, ij_west, ij_east;
+			GMT_LONG j, ij_west, ij_east;
 
 			for (j = 0; j < grd.ny; j++) {	/* For each row, look at west and east bin */
 				ij_west = j * grd.nx;
@@ -589,22 +593,22 @@ int main (int argc, char **argv)
 				n_stuffed++;
 			}
 		}
-		GMT_free ((void *)flag);
 		
 		if (gmtdefs.verbose) {
 			sprintf (line, "%s\n", gmtdefs.d_format);
-			fprintf (stderr, "%s:  n_read: %d  n_used: %d  n_filled: %d  n_empty: %d set to ", GMT_program,
+			fprintf (stderr, "%s:  n_read: %ld  n_used: %ld  n_filled: %ld  n_empty: %ld set to ", GMT_program,
 				n_read, n_used, n_filled, n_empty);
 			(GMT_is_dnan (Ctrl->N.value)) ? fprintf (stderr, "NaN\n") : fprintf (stderr, line, Ctrl->N.value);
-			if (n_bad) fprintf (stderr, "%s: %d records unreadable\n", GMT_program, n_bad);
-			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %d nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
-			if (n_confused) fprintf (stderr, "%s: Warning - %d values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
+			if (n_bad) fprintf (stderr, "%s: %ld records unreadable\n", GMT_program, n_bad);
+			if (n_stuffed && !count) fprintf (stderr, "%s: Warning - %ld nodes had multiple entries that were processed\n", GMT_program, n_stuffed);
+			if (n_confused) fprintf (stderr, "%s: Warning - %ld values gave bad indices: Pixel vs gridline confusion?\n", GMT_program, n_confused);
 		}
 	}
 
 	GMT_err_fail (GMT_write_grd (Ctrl->G.file, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE), Ctrl->G.file);
 
 	GMT_free ((void *)a);
+	if (!Ctrl->E.active) GMT_free ((void *)flag);
 
 	Free_Xyz2grd_Ctrl (Ctrl);	/* Deallocate control structure */
 
@@ -616,7 +620,7 @@ int main (int argc, char **argv)
 void *New_Xyz2grd_Ctrl () {	/* Allocate and initialize a new control structure */
 	struct XYZ2GRD_CTRL *C;
 	
-	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, 1, sizeof (struct XYZ2GRD_CTRL), "New_Xyz2grd_Ctrl");
+	C = (struct XYZ2GRD_CTRL *) GMT_memory (VNULL, (size_t)1, sizeof (struct XYZ2GRD_CTRL), "New_Xyz2grd_Ctrl");
 	
 	/* Initialize values whose defaults are not 0/FALSE/NULL */
 	C->N.value = GMT_d_NaN;
